==> thx-giocaonline-0.1.7.tar.gz: thx-giocaonline-0.1.7/thx/giocaonline/client/base_aams_client_mixin.py <==
# -*- coding: utf-8 -*-
"""AAMS client module"""

import json

import pytz
import logging

from datetime import datetime
from typing import Optional, Iterable, Any, Mapping, List

from requests import Response
from urllib.parse import urljoin

from thx.giocaonline.client.base import BaseClient
from thx.giocaonline.exceptions import AAMSException, ValidationError

logger = logging.getLogger(__name__)
PrizeObject = Mapping[str, float]


class AAMSClientMixin(BaseClient):
    """AAMS HTTP client mixin"""
    current_message: Optional[int] = None
    data: Optional[dict] = None
    base_url: str

    # TO BE DEFINED IN SUB CLASSES
    _methods_map: dict = {}
    endpoints: dict = {}

    # Api method
    _api_method: str = 'post'

    def show_available_methods(self) -> str:
        return "\n".join(
            [f"Code {k}: {v.replace(f'_{k}', '').replace('_', ' ')}" for k, v in self._methods_map.items()]
        )

    def handle_response(self, response: Response) -> Any:
        result = self.handle_message_response(response)
        self.current_message = None
        return result

    def do_request_message(self, message_code: int, **kwargs):
        """Esegue un'azione generica in base al codice"""
        method = self._methods_map.get(message_code)
        if method is None:
            raise NotImplementedError("Method with code %d is not implemented yet" % message_code)
        return getattr(self, method)(**kwargs)

    def _do_request_message(self, message_code: int, **kwargs) -> Any:
        """Metodo che effettua la chiamata corretta a seconda del message_code passato"""
        # set current message
        self.current_message = message_code
        # validate and prepare data
        try:
            self.validate_and_prepare_data(message_code, **kwargs)
            endpoint = self.get_endpoint(message_code)
        except AttributeError:
            raise NotImplementedError(f'validate_and_prepare_data_{message_code} is not implemented yet.')
        except KeyError:
            raise NotImplementedError(f'endpoint for message code %d is not supported yet' % message_code)
        # make request
        url = urljoin(self.base_url, endpoint)
        return self._request_api(self._api_method, url, data=self.data, signed=False)

    def get_endpoint(self, message_code):
        return self.endpoints[message_code]
    
    def validate_and_prepare_data(self, message_code, **kwargs):
        getattr(self, f'validate_and_prepare_data_{message_code}')(**kwargs)

    # RESPONSE HANDLING
    @staticmethod
    def decode_response_body(response: Response) -> dict:
        result = response.content.decode()

        try:
            return json.loads(result)
        except json.JSONDecodeError as e:
            logger.info("Cannot parse JSON response")
            logger.info(result)
            logger.error(e)
            return result

    def handle_message_response(self, response: Response) -> tuple:
        """Gestisce la risposta in base al messaggio della richiesta."""
        return getattr(
            self,
            f"handle_message_{self.current_message}_response",
            "handle_generic_message_response"
        )(response)

    def handle_generic_message_response(self, response: Response) -> tuple:
        raise NotImplementedError()

    @staticmethod
    def get_aams_message_error(error_code: int, *args, **kwargs) -> AAMSException:
        return AAMSException(error_code, *args, **kwargs)

    # VALIDATION
    @staticmethod
    def check_length(field_name: str, text: str, length: int, mode: str = "e"):
        if mode == "e":
            if len(text) != length:
                raise ValidationError(
                    "`%s` len must be equal to %d, but actual length is %d" % (field_name, length, len(text))
                )
        elif mode == "l":
            if len(text) >= length:
                raise ValidationError(
                    "`%s` len must be less than %d, but actual length is %d" % (field_name, length, len(text))
                )
        elif mode == "g":
            if len(text) <= length:
                raise ValidationError(
                    "`%s` len must be grater than %d, but actual length is %d" % (field_name, length, len(text))
                )
        else:
            raise ValueError("Invalid mode %s" % mode)

    @staticmethod
    def check_numeric_value(field_name: str, value: (int, float), check_value: (int, float), mode: str = "e"):
        """Validate numeric values.

        :param field_name: used for validation error message
        :param value: value to be validated
        :param check_value: value on which validate the param `value`
        :param mode: can be: "e" equal, "l" less, "g" grater, "le" less than or equal, "ge" grater than or equal
        """
        if mode == "e":
            if value != check_value:
                raise ValidationError(
                    "`%s` must be equal to %d, but actual value is %d" % (field_name, check_value, value)
                )
        elif mode == "l":
            if value >= check_value:
                raise ValidationError(
                    "`%s` must be less than %d, but actual value is %d" % (field_name, check_value, value)
                )
        elif mode == "le":
            if value > check_value:
                raise ValidationError(
                    "`%s` must be less than or equal to %d, but actual value is %d" % (field_name, check_value, value)
                )
        elif mode == "g":
            if value <= check_value:
                raise ValidationError(
                    "`%s` must be grater than %d, but actual value is %d" % (field_name, check_value, value)
                )
        elif mode == "ge":
            if value < check_value:
                raise ValidationError(
                    "`%s` must be grater than or equal to %d, but actual value is %d" % (field_name, check_value, value)
                )
        else:
            raise ValueError("Invalid mode %s" % mode)

    @staticmethod
    def check_decimal_digits(field_name: str, value: float, decimals: int, mode: str = "e"):
        """Validate float decimal digits.

        :param field_name: used for validation error message
        :param value: value to be validated
        :param decimals: numer of decimals on which validate the param `value`
        :param mode: can be: "e" equal, "l" less, "g" grater, "le" less than or equal, "ge" grater than or equal
        """
        try:
            value = int(str(float(value)).split(".")[1])
        except IndexError:
            # an integer was passed, so set value at 0
            value = 0

        check_value = len(str(value))

        if mode == "e":
            if check_value != decimals:
                raise ValidationError(
                    "`%s` decimals count must be equal to %d, but actual value is %d" % (field_name, decimals, value)
                )
        elif mode == "l":
            if check_value >= decimals:
                raise ValidationError(
                    "`%s` decimals count must be less than %d, but actual value is %d" % (field_name, decimals, value)
                )
        elif mode == "le":
            if check_value > decimals:
                raise ValidationError(
                    "`%s` decimals count must be less than or equal to %d, "
                    "but actual value is %d" % (field_name, decimals, value)
                )
        elif mode == "g":
            if check_value <= decimals:
                raise ValidationError(
                    "`%s` decimals count must be grater than %d, but actual value is %d" % (field_name, decimals, value)
                )
        elif mode == "ge":
            if check_value < decimals:
                raise ValidationError(
                    "`%s` decimals count must be grater than or equal to %d, "
                    "but actual value is %d" % (field_name, decimals, value)
                )
        else:
            raise ValueError("Invalid mode %s" % mode)

    @staticmethod
    def check_timezone(field_name: str, datetime_field: datetime, tz=pytz.utc):
        if not datetime_field.tzinfo == tz:
            raise ValidationError(
                "%s timezone must be set as %s, but is set as %s" % (field_name, tz, datetime_field.tzinfo,)
            )

    @staticmethod
    def check_value_in(field_name: str, value: Any, _in: Iterable):
        if value not in _in:
            raise ValidationError(
                "%s: value %s is not part of %s" % (field_name, value, _in,)
            )

    @staticmethod
    def check_enum(field_name: str, value: Any, enum):
        try:
            enum(value)
        except AttributeError:
            raise ValidationError(
                "%s: value %s is not part of Enum %s" % (field_name, value, enum,)
            )

    @staticmethod
    def check_prizes_list(field_name: str, value: List[PrizeObject], length: int):
        if len(value) > length:
            raise ValidationError(
                "%s: value list is longer than %s" % (field_name, length)
            )

==> thx-giocaonline-0.2.1.tar.gz: thx-giocaonline-0.2.1/thx/giocaonline/client/base_aams_client_mixin.py <==
# -*- coding: utf-8 -*-
"""AAMS client module"""

import json

import pytz
import logging

from datetime import datetime
from typing import Optional, Iterable, Any, Mapping, List

from requests import Response
from urllib.parse import urljoin

from thx.giocaonline.client.base import BaseClient
from thx.giocaonline.exceptions import AAMSException, ValidationError

logger = logging.getLogger('giocaonline')
PrizeObject = Mapping[str, float]


class AAMSClientMixin(BaseClient):
    """AAMS HTTP client mixin"""
    current_message: Optional[int] = None
    data: Optional[dict] = None
    base_url: str

    # TO BE DEFINED IN SUB CLASSES
    _methods_map: dict = {}
    endpoints: dict = {}

    # Api method
    _api_method: str = 'post'

    def show_available_methods(self) -> str:
        return "\n".join(
            [f"Code {k}: {v.replace(f'_{k}', '').replace('_', ' ')}" for k, v in self._methods_map.items()]
        )

    def handle_response(self, response: Response) -> Any:
        result = self.handle_message_response(response)
        self.current_message = None
        return result

    def do_request_message(self, message_code: int, **kwargs):
        """Esegue un'azione generica in base al codice"""
        method = self._methods_map.get(message_code)
        if method is None:
            raise NotImplementedError("Method with code %d is not implemented yet" % message_code)
        return getattr(self, method)(**kwargs)

    def _do_request_message(self, message_code: int, **kwargs) -> Any:
        """Metodo che effettua la chiamata corretta a seconda del message_code passato"""
        # set current message
        self.current_message = message_code
        # validate and prepare data
        try:
            self.validate_and_prepare_data(message_code, **kwargs)
            endpoint = self.get_endpoint(message_code)
        except AttributeError:
            raise NotImplementedError(f'validate_and_prepare_data_{message_code} is not implemented yet.')
        except KeyError:
            raise NotImplementedError(f'endpoint for message code %d is not supported yet' % message_code)
        # make request
        url = urljoin(self.base_url, endpoint)
        return self._request_api(self._api_method, url, json=self.data, signed=False)

    def get_endpoint(self, message_code):
        return self.endpoints[message_code]
    
    def validate_and_prepare_data(self, message_code, **kwargs):
        getattr(self, f'validate_and_prepare_data_{message_code}')(**kwargs)

    # RESPONSE HANDLING
    @staticmethod
    def decode_response_body(response: Response) -> dict:
        result = response.content.decode()

        try:
            return json.loads(result)
        except json.JSONDecodeError as e:
            logger.info("Cannot parse JSON response")
            logger.info(result)
            logger.error(e)
            return result

    def handle_message_response(self, response: Response) -> tuple:
        """Gestisce la risposta in base al messaggio della richiesta."""
        return getattr(
            self,
            f"handle_message_{self.current_message}_response",
            "handle_generic_message_response"
        )(response)

    def handle_generic_message_response(self, response: Response) -> tuple:
        raise NotImplementedError()

    @staticmethod
    def get_aams_message_error(error_code: int, *args, **kwargs) -> AAMSException:
        return AAMSException(error_code, *args, **kwargs)

    # VALIDATION
    @staticmethod
    def check_length(field_name: str, text: str, length: int, mode: str = "e"):
        if mode == "e":
            if len(text) != length:
                raise ValidationError(
                    "`%s` len must be equal to %d, but actual length is %d" % (field_name, length, len(text))
                )
        elif mode == "l":
            if len(text) >= length:
                raise ValidationError(
                    "`%s` len must be less than %d, but actual length is %d" % (field_name, length, len(text))
                )
        elif mode == "g":
            if len(text) <= length:
                raise ValidationError(
                    "`%s` len must be grater than %d, but actual length is %d" % (field_name, length, len(text))
                )
        else:
            raise ValueError("Invalid mode %s" % mode)

    @staticmethod
    def check_numeric_value(field_name: str, value: (int, float), check_value: (int, float), mode: str = "e"):
        """Validate numeric values.

        :param field_name: used for validation error message
        :param value: value to be validated
        :param check_value: value on which validate the param `value`
        :param mode: can be: "e" equal, "l" less, "g" grater, "le" less than or equal, "ge" grater than or equal
        """
        if mode == "e":
            if value != check_value:
                raise ValidationError(
                    "`%s` must be equal to %d, but actual value is %d" % (field_name, check_value, value)
                )
        elif mode == "l":
            if value >= check_value:
                raise ValidationError(
                    "`%s` must be less than %d, but actual value is %d" % (field_name, check_value, value)
                )
        elif mode == "le":
            if value > check_value:
                raise ValidationError(
                    "`%s` must be less than or equal to %d, but actual value is %d" % (field_name, check_value, value)
                )
        elif mode == "g":
            if value <= check_value:
                raise ValidationError(
                    "`%s` must be grater than %d, but actual value is %d" % (field_name, check_value, value)
                )
        elif mode == "ge":
            if value < check_value:
                raise ValidationError(
                    "`%s` must be grater than or equal to %d, but actual value is %d" % (field_name, check_value, value)
                )
        else:
            raise ValueError("Invalid mode %s" % mode)

    @staticmethod
    def check_decimal_digits(field_name: str, value: float, decimals: int, mode: str = "e"):
        """Validate float decimal digits.

        :param field_name: used for validation error message
        :param value: value to be validated
        :param decimals: numer of decimals on which validate the param `value`
        :param mode: can be: "e" equal, "l" less, "g" grater, "le" less than or equal, "ge" grater than or equal
        """
        try:
            value = int(str(float(value)).split(".")[1])
        except IndexError:
            # an integer was passed, so set value at 0
            value = 0

        check_value = len(str(value))

        if mode == "e":
            if check_value != decimals:
                raise ValidationError(
                    "`%s` decimals count must be equal to %d, but actual value is %d" % (field_name, decimals, value)
                )
        elif mode == "l":
            if check_value >= decimals:
                raise ValidationError(
                    "`%s` decimals count must be less than %d, but actual value is %d" % (field_name, decimals, value)
                )
        elif mode == "le":
            if check_value > decimals:
                raise ValidationError(
                    "`%s` decimals count must be less than or equal to %d, "
                    "but actual value is %d" % (field_name, decimals, value)
                )
        elif mode == "g":
            if check_value <= decimals:
                raise ValidationError(
                    "`%s` decimals count must be grater than %d, but actual value is %d" % (field_name, decimals, value)
                )
        elif mode == "ge":
            if check_value < decimals:
                raise ValidationError(
                    "`%s` decimals count must be grater than or equal to %d, "
                    "but actual value is %d" % (field_name, decimals, value)
                )
        else:
            raise ValueError("Invalid mode %s" % mode)

    @staticmethod
    def check_timezone(field_name: str, datetime_field: datetime, tz=pytz.utc):
        if not datetime_field.tzinfo == tz:
            raise ValidationError(
                "%s timezone must be set as %s, but is set as %s" % (field_name, tz, datetime_field.tzinfo,)
            )

    @staticmethod
    def check_value_in(field_name: str, value: Any, _in: Iterable):
        if value not in _in:
            raise ValidationError(
                "%s: value %s is not part of %s" % (field_name, value, _in,)
            )

    @staticmethod
    def check_enum(field_name: str, value: Any, enum):
        try:
            enum(value)
        except AttributeError:
            raise ValidationError(
                "%s: value %s is not part of Enum %s" % (field_name, value, enum,)
            )

    @staticmethod
    def check_prizes_list(field_name: str, value: List[PrizeObject], length: int):
        if len(value) > length:
            raise ValidationError(
                "%s: value list is longer than %s" % (field_name, length)
            )
